

Selex <- function(Lengths,Selex50, Delta)
{
 Selex <- 1.0/(1.0+exp(-log(19)*(Lengths-Selex50)/Delta))  
 return(Selex)  
}  

# =====================================================================================

Growth <- function(MidLen,EndLen,GrowInc1 ,GrowInc2,Molt1,Molt2,SigmaV,Jump=10)
{
 # Extract 
 nlen <- length(MidLen)
 RefLen1 <- MidLen[2]  
 RefLen2 <- MidLen[nlen-1] 
 
 # Compute temp variables
 BetaA = (GrowInc2 - GrowInc1)/(RefLen2-RefLen1)
 AlphaA =  GrowInc1 - BetaA*RefLen1
 Zed1 = log(1.0/Molt1-1.0)
 Zed2 = log(1.0/Molt2-1.0)
 AlphaB = (Zed2-Zed1)/(RefLen2-RefLen1)
 BetaB = -1.0*(Zed1/AlphaB-RefLen1)
  
 # Create output objectives
 ProbMolt <- rep(0,nlen)
 TransX <- matrix(0,ncol=nlen,nrow=nlen)
 ExpectedSize <- rep(0,nlen)
 
 # Do for each "for" size-class  
 for (Ilen in 1:(nlen-1))
  {
   LenMid2 <- MidLen[Ilen];
   Len <- LenMid2 + (AlphaA + BetaA*LenMid2)
   Probb <- 1.0/(1.0+exp(AlphaB*(LenMid2-BetaB)));
   ProbMolt[Ilen] <- Probb
   
   Total <- 0;
   nlen2 <- Ilen+Jump-1;
   if (nlen2 > nlen) nlen2 = nlen
   for (Jlen in Ilen:(nlen2-1))
    {
     LenUpp = EndLen[Jlen]
     Zstat = (log(LenUpp)-log(Len))/SigmaV
     Value = pnorm(Zstat);
     if (Jlen==Ilen)
       TransX[Jlen,Ilen] <- (1-Probb) + Probb*(Value - Total)
     else 
       TransX[Jlen,Ilen] <- Probb*(Value - Total)
     Total = Value
    }
   TransX[nlen2,Ilen] <- Probb*(1.0 - Total)
   for (Jlen in Ilen:nlen2)
    ExpectedSize[Ilen] <- ExpectedSize[Ilen] + TransX[Jlen,Ilen]*MidLen[Jlen]
  }
 TransX[nlen,nlen] <- 1.0
 ProbMolt[nlen] <- 0
 ExpectedSize[nlen] <- MidLen[nlen]

 Outs <- NULL
 Outs$TransX <- TransX
 Outs$ProbMolt <- ProbMolt
 Outs$ExpectedSize <- ExpectedSize
 return(Outs)
}   

# =====================================================================================

EqnF <- function(MidLen,TransX,Recr,Selex,Retain,M=0.2,Fval=0,DiscSurv=0,LWE,Lmat)
 {
  # Extract 
  nlen <- length(MidLen)
  Selex2 <- Selex*(Retain + (1-Retain)*(1-DiscSurv))

  # Indentity matrix   
  I <- matrix(0,ncol=nlen,nrow=nlen)
  diag(I) <- 1
  
  # Survival matrix
  S <- matrix(0,ncol=nlen,nrow=nlen)
  diag(S) <- exp(-M)*exp(-Selex2*Fval)
  
  # Solve for N and check
  N <- solve(I-TransX%*%S)%*%Recr
  Cdeck <- N*exp(-M/2)*(1-exp(-Selex2*Fval))
  Ccat <- Cdeck*Retain/((Retain + (1-Retain)*(1-DiscSurv)))
  SSB <- 0
  for (II in 1:nlen)
   if (MidLen[II] > Lmat) SSB <- SSB + 0.001*MidLen[II]^LWE*N[II]
  CatW <- 0
  for (II in 1:nlen) CatW <- CatW + 0.001*MidLen[II]^LWE*Ccat[II]
      
  Outs <- NULL
  Outs$N <- N
  Outs$Cdeck <- Cdeck
  Outs$Ccat <- Ccat
  Outs$FSurv <- exp(-Selex2*Fval)
  Outs$TotalC <- CatW
  Outs$SSB <- SSB
  return(Outs)
 }

# =====================================================================================

SSBPR <- function(MidLen,TransX,Recr,Selex2,Retain,M=0.2,InitFval=0,LWE,Lmat,nlen,I)
 {
  S <- matrix(0,ncol=nlen,nrow=nlen)
  diag(S) <- exp(-M)*exp(-Selex2*InitFval)
  
  # Solve for N and then acocunt for initial depletion
  NInit <- solve(I-TransX%*%S)%*%Recr
  SSB <- 0
  for (II in 1:nlen)
    if (MidLen[II] > Lmat) SSB <- SSB + 0.001*MidLen[II]^LWE*NInit[II]

  Outs <- NULL
  Outs$SSBPR <- SSB
  Outs$N <- NInit
  return(Outs)
 }  
  
# =====================================================================================

AllProject <- function(MidLen,TransX,Recr,Selex,Retain,M=0.2,InitFval=0,DiscSurv=0,LWE,Lmat,Steep,SigmaR,prowR,SigmaB,prowB,InitDepl,SSBtarg,LowB,HighB,HCRType,Nproj=100,Nsim=20)
{
 #Steep <- 0.8
 #SigmaR <- 0.6
 #prowR <- 0.7
 #SigmaB <- 0.2
 #prowB <- 0.9
 #InitDepl <- 0.2
 #SSBtarg <- 0.35
 #LowB <- 0.2
 #HighB <- 0.35

 TheSeeds <- floor(runif(Nsim,1000,100000))

 SSB <- matrix(0,nrow=Nsim,ncol=Nproj+1)
 CatW <- matrix(0,nrow=Nsim,ncol=Nproj)
 
 for (Isim in 1:Nsim) 
  {
   Outs <- Project(MidLen,TransX,Recr,Selex,Retain,M=0.2,InitFval=0,DiscSurv=0,LWE,Lmat,Nproj,TheSeeds[Isim],
      Steep=Steep,SigmaR=SigmaR,prowR=prowR,SigmaB=SigmaB,prowB=prowB,InitDepl=InitDepl,
      SSBtarg=SSBtarg,LowB=LowB,HighB=HighB,HCRType=HCRType)
   SSB[Isim,]  <-  Outs$SSB
   CatW[Isim,]  <-  Outs$CatW
  } 

 par(mfrow=c(2,2))
 ymaxS <- max(100,SSB)*1.05
 ymaxC <- max(1,CatW)*1.05
 Years1 <- 0:Nproj
 Years2 <- 0:(Nproj-1)
 plot(Years1,SSB[1,],xlab="Year",ylab=expression(SSB/SSB[0]),ylim=c(0,ymaxS),type='l',lty=1)
 for (Isim in 2:5)
  lines(Years1,SSB[Isim,],lty=Isim) 
 plot(Years2,CatW[1,],xlab="Year",ylab="Catch",ylim=c(0,ymaxC),type='l',lty=1)
 for (Isim in 2:5)
  lines(Years2,CatW[Isim,],lty=Isim) 
 
 quants <- matrix(0,nrow=Nproj+1,ncol=5)
 for (Iyear in 1:(Nproj+1))
  quants[Iyear,]  <- quantile(SSB[,Iyear],probs=c(0.05,0.25,0.5,0.75,0.95))

 plot(Years1,SSB[1,],xlab="Year",ylab=expression(SSB/SSB[0]),ylim=c(0,ymaxS),type='n',lty=1)
 xx <- c(Years1,rev(Years1))
 yy <- c(quants[,1],rev(quants[,5]))
 polygon(xx,yy,col="gray50")
 yy <- c(quants[,2],rev(quants[,4]))
 polygon(xx,yy,col="gray95")
 lines(Years1,quants[,3],lwd=2)
 #abline(h=SSBtarg*100,lwd=2)
 
 quants <- matrix(0,nrow=Nproj,ncol=5)
 for (Iyear in 1:(Nproj))
  quants[Iyear,]  <- quantile(CatW[,Iyear],probs=c(0.05,0.25,0.5,0.75,0.95))
 
 plot(Years2,CatW[1,],xlab="Year",ylab="Catch",ylim=c(0,ymaxC),type='n',lty=1)
 xx <- c(Years2,rev(Years2))
 yy <- c(quants[,1],rev(quants[,5]))
 polygon(xx,yy,col="gray50")
 yy <- c(quants[,2],rev(quants[,4]))
 polygon(xx,yy,col="gray95")
 lines(Years2,quants[,3],lwd=2)
}

# =====================================================================================

Project <- function(MidLen,TransX,Recr,Selex,Retain,M=0.2,InitFval=0,DiscSurv=0,LWE,Lmat,Nproj,TheSeed,
                    Steep,SigmaR,prowR,SigmaB,prowB,InitDepl,SSBtarg,LowB,HighB,HCRType)
{
  # Set parameters
  set.seed(TheSeed)
  
  # Recruitment deviations
  Eps <- rep(0,Nproj+1)
  Eps <- rnorm(1,0,SigmaR)
  for (Iyear in 2:(Nproj+1))
   Eps[Iyear] <- prowR*Eps[Iyear-1] + sqrt(1.0-prowR)*rnorm(1,0,SigmaR)
  for (Iyear in 1:(Nproj+1))
   Eps[Iyear] <- exp(Eps[Iyear]-SigmaR^2/2.0)
    
  # Biomass estmation deviations
  EpsB <- rep(0,Nproj+1)
  EpsB <- rnorm(1,0,SigmaB)
  for (Iyear in 2:(Nproj+1))
   EpsB[Iyear] <- prowB*EpsB[Iyear-1] + sqrt(1.0-prowB)*rnorm(1,0,SigmaB)
  for (Iyear in 1:(Nproj+1))
   EpsB[Iyear] <- exp(EpsB[Iyear]-SigmaB^2/2.0)
  EpsB0 <- exp(rnorm(1,0,SigmaB)-SigmaB^2/2.0)
  
  # Extract 
  nlen <- length(MidLen)
  Selex2 <- Selex*(Retain + (1-Retain)*(1-DiscSurv))
  
  # Indentity matrix   
  I <- matrix(0,ncol=nlen,nrow=nlen)
  diag(I) <- 1
  
  # Solve for virgin biomass
  # ========================
  SSB0 <- SSBPR(MidLen,TransX,Recr,Selex2,Retain,M,InitFval=0,LWE,Lmat,nlen,I)$SSBPR
  
  # Solve for target F
  Fmin <- 0; Fmax <- 1000
  for (II in 1:50)
   {
    Ftarg <- (Fmin+Fmax)/2.0 
    SSBR <- SSBPR(MidLen,TransX,Recr,Selex2,Retain,M,InitFval=Ftarg,LWE,Lmat,nlen,I)$SSBPR
    if (SSBR < SSBtarg*SSB0) Fmax <- Ftarg else Fmin <- Ftarg
   }  
  #cat(Ftarg,SSBR/SSB0,"\n")

  # Solve for initial state
  Fmin <- 0; Fmax <- 1000
  for (II in 1:50)
   {
    Finit <- (Fmin+Fmax)/2.0 
    Outs <- SSBPR(MidLen,TransX,Recr,Selex2,Retain,M,InitFval=Finit,LWE,Lmat,nlen,I)
    SSBX <- Outs$SSBPR
    Recruit <- (4*Steep*SSBX/SSB0 - (1-Steep))/((5*Steep-1)*SSBX/SSB0)
    SSBX <- SSBX * Recruit
    if (SSBX < InitDepl*SSB0) Fmax <- Finit else Fmin <- Finit
   }  
  #cat(Finit,SSBX/SSB0,"\n")

  # Outputs
  SSB <- rep(0,Nproj+1)
  SSBOBS <- rep(0,Nproj+1)
  CatW <- rep(0,Nproj)
  N <- matrix(0,nrow=(Nproj+1),ncol=nlen)
  S <- matrix(0,ncol=nlen,nrow=nlen)
  N[1,] <- Outs$N*Recruit
  SSB[1] <- SSBX
  
  # Now project
  for (Iyear in 1:Nproj)
   {
    for (II in 1:Nproj) SSBOBS[II] <- SSB[II]*EpsB[II]
    Depl <- SSBOBS[Iyear] / (SSB0*EpsB0)
    if (HCRType=="thresh")
     {  
      if (Depl < LowB)
       Fval <- 0
      else
       if (Depl < HighB)
        Fval <- Ftarg * (Depl-LowB)/(HighB-LowB)  
       else
        Fval <- Ftarg
     } 
    if (HCRType=="constE") Fval <- Ftarg
    #cat(Iyear,Depl,Fval,SSB[Iyear]/SSB0,"\n")
    
    diag(S) <- exp(-M)*exp(-Selex2*Fval)
    N[Iyear+1,] <- TransX%*%(S%*%N[Iyear,])

    Cdeck <- N[Iyear,]*exp(-M/2)*(1-exp(-Selex2*Fval))
    Ccat <- Cdeck*Retain/((Retain + (1-Retain)*(1-DiscSurv)))
    CatW[Iyear] <- 0
    for (II in 1:nlen) CatW[Iyear] <- CatW[Iyear] + 0.001*MidLen[II]^LWE*Ccat[II]

    SSB[Iyear+1] <- 0
    for (II in 1:nlen)
      if (MidLen[II] > Lmat) SSB[Iyear+1] <- SSB[Iyear+1] + 0.001*MidLen[II]^LWE*N[Iyear+1,II]
    Recruit <- 4*Steep*SSB[Iyear+1]/SSB0/((1-Steep)+(5*Steep-1)*SSB[Iyear+1]/SSB0)*Eps[Iyear]
    N[Iyear+1,] <- N[Iyear+1,] + Recr*Recruit
   }  

  Outs <- NULL
  Outs$SSB <- SSB/SSB0*100
  Outs$CatW <- CatW
  return(Outs)
}


# =====================================================================================

Yields <- function(MidLen,TransX,Recs,Selex,Retain,M=0.2,InitFval=0,DiscSurv=0,LWE,Lmat,Steep,Fmax)
{
  Eqn <- EqnF(MidLen,TransX,Recs,Selex,Retain,M=M,Fval=0,LWE=LWE,Lmat=Lmat)
  SSB0 <- Eqn$SSB
  
  Fmin <- 0
  Fmax2 <- 5
  for (II in 1:30)
   {
    FvalF <- (Fmin+Fmax2)/2
    Eqn <- EqnF(MidLen,TransX,Recs,Selex,Retain,M=M,Fval=FvalF,LWE=LWE,Lmat=Lmat)
    SSB <- Eqn$SSB/SSB0
    Recr <- (4*Steep*SSB - (1-Steep))/((5*Steep-1)*SSB)
    if (Recr > 0) Fmin <- FvalF else Fmax2 <- FvalF
   }  
  Finc <- FvalF/100
  Nstep <- max(40,Fmax / Finc)

  Fvals <- rep(0,Nstep+1)      
  TotalC <- rep(0,Nstep+1)
  SSB <- rep(0,Nstep+1)
  for (II in 0:Nstep)
  {
    FvalF <- II/(Nstep)*Fmax
    Eqn <- EqnF(MidLen,TransX,Recs,Selex,Retain,M=M,Fval=FvalF,LWE=LWE,Lmat=Lmat)
    Fvals[II+1] <- FvalF
    TotalC[II+1] <- Eqn$TotalC
    SSB[II+1] <- Eqn$SSB
  }  
  SSB <- SSB/SSB[1]
  Recr <- (4*Steep*SSB - (1-Steep))/((5*Steep-1)*SSB)
  for (II in 0:Nstep) if (Recr[II+1] < 0) Recr[II+1] <- 0

  par(mfrow=c(3,2))
  ymax <- max(TotalC)*1.05
  plot(Fvals,TotalC,xlab="Fishing mortality",ylab="Yield-per-recruit",type='l',lty=1,ylim=c(0,ymax),yaxs="i",xaxs="i",cex.lab=1.5,cex.axis=1.6,lwd=2)
  ymax <- max(TotalC*Recr)*1.05
  plot(Fvals,TotalC*Recr,xlab="Fishing mortality",ylab="Yield",type='l',lty=1,ylim=c(0,ymax),yaxs="i",xaxs="i",cex.lab=1.5,cex.axis=1.6,lwd=2)
  ymax <- max(SSB)*1.05
  plot(Fvals,SSB,xlab="Fishing mortality",ylab="Spawning biomass-per-recruit",type='l',lty=1,ylim=c(0,ymax),yaxs="i",xaxs="i",cex.lab=1.5,cex.axis=1.6,lwd=2)
  ymax <- max(SSB*Recr)*1.05
  plot(Fvals,SSB*Recr,xlab="Fishing mortality",ylab="Spawning biomass",type='l',lty=1,ylim=c(0,ymax),yaxs="i",xaxs="i",cex.lab=1.5,cex.axis=1.6,lwd=2)
  ymax <- max(TotalC*Recr)*1.05
  xmax <- max(SSB*Recr)*1.05
  plot(SSB*Recr,TotalC*Recr,xlab="Spawning biomass",ylab="Yield",type='l',lty=1,ylim=c(0,ymax),yaxs="i",xaxs="i",cex.lab=1.5,cex.axis=1.6,lwd=2)
  plot(c(SSB*Recr,0),c(Recr,0),xlab="Spawning biomass",ylab="Recruitment",type='l',lty=1,ylim=c(0,1.05),yaxs="i",xaxs="i",cex.lab=1.5,cex.axis=1.6,lwd=2)
}  

# =====================================================================================


SelectScenario <- function()
{
 Parms <- NULL
 Parms$Nyear <- 100
 Parms$SigmaB <- 0.02
 Parms$SigmaE <- 0.2
 Parms$Prow <- 0.7
 Parms$SigmaBest <- 0.2
 Parms$ProwEst <- 0.7
 Parms$K <- 10000
 Parms$r <- 0.2
 Parms$Cyear <- 40
 Parms$Eps <- rep(0,length=Parms$Nyear+1)
 Parms$BEps <- rep(0,length=Parms$Nyear+1)
 Parms$Effort <- rep(0,length=Parms$Nyear)
 Parms$Catch <- rep(0,length=Parms$Nyear)
 Parms$F <- rep(0,length=Parms$Nyear)
 Parms$FEst <- rep(0,length=Parms$Nyear)
 Parms$Biomass <- rep(0,length=Parms$Nyear+1)
 Parms$BiomassEst <- rep(0,length=Parms$Nyear+1)
 return(Parms)  
}

# ======================================================================================

InitialModel <- function(Parms,seed)
{
  set.seed(seed) 
  
  # Devs in   
  Parms$Eps[1] <- rnorm(1,0,Parms$SigmaB)
  for (II in 1:Parms$Nyear)
    Parms$Eps[II+1] <- Parms$Prow*Parms$Eps[II]+sqrt(1-Parms$Prow^2)*rnorm(1,0,Parms$SigmaB)
  for (II in 1:(Parms$Nyear+1)) Parms$Eps[II] <- Parms$Eps[II] - Parms$SigmaB^2/2.0
  Parms$BEps[1] <- rnorm(1,0,Parms$SigmaBest)
  for (II in 1:Parms$Nyear)
    Parms$BEps[II+1] <- Parms$ProwEst*Parms$BEps[II]+sqrt(1-Parms$ProwEst^2)*rnorm(1,0,Parms$SigmaBest)
  for (II in 1:(Parms$Nyear+1)) Parms$BEps[II] <- Parms$BEps[II] - Parms$SigmaBest^2/2.0
  
  Parms$Effort[1:10] <- seq(from=0,to=1,length=10)
  Parms$Effort[11:40] <- 1
  for (Iyear in 1:40) Parms$Effort[Iyear] <- Parms$Effort[Iyear]*exp(rnorm(1,0,Parms$SigmaE)-Parms$SigmaE^2/2)
  
  Parms$Biomass[1] <- Parms$K
  qmin <- 0
  qmax <- 5
  for (Itune in 1:15)
  {
    qbar <- (qmin+qmax)/2
    for (Iyear in 1:40)
    { 
      Parms$F[Iyear] <- qbar*Parms$Effort[Iyear]
      Parms$Catch[Iyear] <- Parms$F[Iyear]*Parms$Biomass[Iyear]
      Production <- Parms$r*Parms$Biomass[Iyear]*(1-Parms$Biomass[Iyear]/Parms$K)
      Parms$Biomass[Iyear+1] <- max(0.01,Parms$Biomass[Iyear] + Production-Parms$Catch[Iyear])
      Parms$Biomass[Iyear+1] <- Parms$Biomass[Iyear+1]*exp(Parms$Eps[Iyear+1])
    } 
    if (Parms$Biomass[41]/Parms$K > 0.4) qmin <- qbar else qmax <- qbar
    #cat(qbar,Parms$Biomass[41]/Parms$K,"\n")
  } 
  
  return(Parms)  
}  
# ======================================================================================

UpdateDynamics<-function(Parms,TAC,Iyear)
{
  # Biomass  after production
  NextBio <- Parms$Biomass[Iyear] + Parms$r*Parms$Biomass[Iyear]*(1-Parms$Biomass[Iyear]/Parms$K)
  # Check for catch that is too high
  if (TAC > 0.95*NextBio) TheCatch <- 0.95*NextBio else TheCatch <- TAC
  
  # Update the dynamics
  Parms$F[Iyear] <- TheCatch / Parms$Biomass[Iyear]
  Parms$Catch[Iyear] <- TheCatch
  Parms$Biomass[Iyear+1] <- (NextBio - Parms$Catch[Iyear])*exp(Parms$Eps[Iyear+1])
  return(Parms)
}
# ======================================================================================

BreakCtl <- function(Parms,Low=0.1,High=0.4,Fmult=0.5,HCRtype,ConstC,ConstE,Nsim=10)
{  
 BiomassOut <- matrix(0,ncol=Parms$Nyear+1,nrow=Nsim)
 CatchOut <- matrix(0,ncol=Parms$Nyear,nrow=Nsim)

 for (Isim in 1:Nsim)  
  {   
  
   Iseed <- (Isim-1)*10+666
   # Set up the model
   Parms <- InitialModel(Parms,Iseed)
   for (Iyear in 41:100)
    {  
     for (Jyear in 1:(Parms$Nyear)) Parms$BiomassEst[Jyear] <- Parms$Biomass[Jyear]*exp(Parms$BEps[Jyear])
     MSY <- Parms$r*Parms$K/4.0
     Khat <- Parms$BiomassEst[1]
     BmsyHat <- Khat/2
     rhat <- 4*MSY/Khat
     CurrentDepl <- Parms$BiomassEst[Iyear]/BmsyHat
    
     # Set the TAC
     if (HCRtype=="thresh")
      { 
       if (CurrentDepl < Low) 
        TAC <- 0
       else
        {  
         if (CurrentDepl < High)
          Hrate <- Fmult*rhat/2*(CurrentDepl-Low)/(High-Low)
         else
          Hrate <- Fmult*rhat/2
         TAC <- Hrate*Parms$BiomassEst[Iyear]   
        }    
      } 
     if (HCRtype=="constC") TAC <- ConstC
     if (HCRtype=="constE") TAC <- ConstE*Parms$BiomassEst[Iyear]

     # update dynamics
     Parms <- UpdateDynamics(Parms,TAC,Iyear)  
    }
  
   # save the results
   BiomassOut[Isim, ] <- Parms$Biomass
   CatchOut[Isim, ] <- Parms$Catch
 }
 Outs <- NULL
 Outs$r <- Parms$r
 Outs$Nsim <- Nsim
 Outs$Nyear <- 100
 Outs$BiomassOut <- BiomassOut
 Outs$CatchOut <- CatchOut
 Outs$Parms <- Parms
 return(Outs)
}

# ======================================================================================

CalculateOutputs<- function(Outs,Nyear,BLIM,CLIM)
 {
  r <- Outs$r
  Nsim <- Outs$Nsim
  Biomass <- Outs$BiomassOut
  Catch <-Outs$CatchOut
  Expl <- matrix(0,nrow=Nsim,ncol=Nyear)
  for (Isim in 1:Nsim)
   for (Iyear in 1:Nyear) Expl[Isim,Iyear] <- Catch[Isim,Iyear]/Biomass[Isim,Iyear]/(r/2.0)
  for (Isim in 1:Nsim)
   Biomass[Isim,] <- Biomass[Isim,]/(0.5*Biomass[Isim,1])*100
  
  # proportion in the green zones
  CntGreen <- 0
  for (Isim in 1:Nsim)
   for (Iyear in 41:Nyear)  
    if (Expl[Isim,Iyear]  < 1 & Biomass[Isim,Iyear] > 100) CntGreen <- CntGreen + 1
  ProbGreen <- round(CntGreen/(Nsim*(Nyear-41+1)),3)  

  CntSafe <- 0
  for (Isim in 1:Nsim)
    for (Iyear in 41:Nyear)  
      if (Biomass[Isim,Iyear] > 100) CntSafe <- CntSafe + 1
  ProbSafe <- round(CntSafe/(Nsim*(Nyear-41+1)),3)  

  CntBLIM <- 0
  for (Isim in 1:Nsim)
    for (Iyear in 41:Nyear)  
      if (Biomass[Isim,Iyear] > BLIM*100) CntBLIM <- CntBLIM + 1
  ProbBLIM <- round(CntBLIM/(Nsim*(Nyear-41+1)),3)  

  CntCLIM <- 0
  for (Isim in 1:Nsim)
    for (Iyear in 41:Nyear)  
      if (Catch[Isim,Iyear] > CLIM) CntCLIM <- CntCLIM + 1
  ProbCLIM <- round(CntCLIM/(Nsim*(Nyear-41+1)),3)  
  
  AAV1 = 0; AAV2 <- 0; CatAve <- rep(0,Nsim);AAV <- rep(0,Nsim)
  for (Isim in 1:Nsim)
   {  
    for (Iyear in (Nyear-4):(Nyear)) CatAve[Isim] <- CatAve[Isim] + Catch[Isim,Iyear]
    CatAve[Isim] <- CatAve[Isim] / 5.0
    for (Iyear in 41:(Nyear-1))
     {
      AAV1 <- AAV1 + Catch[Isim,Iyear]
      AAV2 <- AAV2 + abs(Catch[Isim,Iyear]-Catch[Isim,Iyear+1])
     }  
    AAV[Isim] <- AAV2/max(AAV1,0.001)
   }    
  
    
  # median and 90% intervals for final-year biomass, catch and exploitation rate
  PercB <- as.double(quantile(Biomass[,Nyear+1],probs=c(0.05,0.5,0.95)))
  PercC <- as.double(quantile(CatAve,probs=c(0.05,0.5,0.95)))
  PercE <- as.double(quantile(Expl[,Nyear],probs=c(0.05,0.5,0.95)))
  
  # Record the outputs
  SumStat <- NULL
  SumStat$PercB <- PercB
  SumStat$PercC <- PercC
  SumStat$PercE <- PercE
  SumStat$ProbGreen <- ProbGreen
  SumStat$ProbSafe <- ProbSafe
  SumStat$ProbBLIM <- ProbBLIM
  SumStat$ProbCLIM <- ProbCLIM
  SumStat$AAV <- round(median(AAV),3)
  return(SumStat)
}

