
require(shiny)
require(markdown)
require(ggmap)
require(ggplot2)
require(devtools)
require(shinydashboard)


source("grafs.r")
source("andrefile.r")
#source("HerringScenariosFINWeb.r")
#source("Herring Sim Function Web.r")
#source("https://raw.githubusercontent.com/SNAPwg/SNAP/master/Master.R")

## ------------------------------------------------------------------------------------ ##
## Define server logic required to run scripts
## ------------------------------------------------------------------------------------ ##
defaultVals<-c(0.1,.1,.3,.5,.5)
NumberOfOptions <- 1

#---Fisheries Glossary
glossary <- read.csv("www/gloss.csv", header=F)
glossary <- glossary[1:23,]
colnames(glossary) <- c("Term", "Definition")

#---

function(input, output, session) {
  
   # generic stuff
   MidLen <-seq(from=25,to=200,by=5)
   nlen <- length((MidLen))
   EndLen <- MidLen + 0.5
  
  
     # Start by setting up the scenario
   Iseed <- 666
   Parms <- SelectScenario()
   Parms2 <- SelectScenario()

   values <- reactiveValues()
   values$df <- data.frame(HCRType=NA,Hrate = NA,CutOff1 = NA,CutOff2 = NA,ConCatch=NA,ConExpl=NA,MedCatch = NA, MedBio = NA,
                           PGreen = NA,PBMSY=NA,PBLIM=NA,PCLIM=NA,AAV=NA)
   values$full <- data.frame(LowB = NA,MedB = NA,UppB = NA,LowC = NA,MedC = NA,UppC = NA,LowE = NA,MedE = NA,UppE = NA)
   values$Outs <- NULL
   values$Single  <- isolate(InitialModel(Parms2,input$seed))
   values$BioTraj <- NULL
   values$CatTraj <- NULL
   values$Selex <- NULL
   values$Retain <- NULL

   # Do a control rule 
   output$CTL2 <- renderPlot({ ControlRuleFplot(input$HCR.range[1],input$HCR.range[2],input$HCR.prop2) })

   # Do a projection
   observeEvent(input$do,{
     isolate(values$Outs <- BreakCtl(Parms,input$HCR.range[1],input$HCR.range[2],input$HCR.prop2,input$HCR.type,input$HCR.cat,input$HCR.expl,input$Nsim))
     a <- CalculateOutputs(values$Outs,input$Nyear,input$BLIM,input$CLIM)
     if (input$HCR.type=="thresh")
      newLine <- isolate(c("Threshold",input$HCR.prop2,input$HCR.range[1],input$HCR.range[2],NA,NA,round(a$PercC[2],1),round(a$PercB[2],1),a$ProbGreen,a$ProbSafe,a$ProbBLIM,a$ProbCLIM,a$AAV))
     if (input$HCR.type=="constC")
       newLine <- isolate(c("Constant Catch",NA,NA,NA,input$HCR.cat,NA,round(a$PercC[2],1),round(a$PercB[2],1),a$ProbGreen,a$ProbSafe,a$ProbBLIM,a$ProbCLIM,a$AAV))
     if (input$HCR.type=="constE")
       newLine <- isolate(c("Constant Exploitation",NA,NA,NA,NA,input$HCR.expl,round(a$PercC[2],1),round(a$PercB[2],1),a$ProbGreen,a$ProbSafe,a$ProbBLIM,a$ProbCLIM,a$AAV))
     isolate(values$df <- rbind(values$df, newLine))
     isolate(values$BioTraj <- rbind(values$BioTraj,values$Outs$BiomassOut) )
     isolate(values$CatTraj <- rbind(values$CatTraj,values$Outs$CatchOut) )
     newLine <- isolate(c(a$PercB,a$PercC,a$PercE))
     isolate(values$full <- rbind(values$full, newLine))
     NumberOfOptions <- NumberOfOptions + 1
   })

   observeEvent(input$doMHCR,{
     if (input$HCR2.type=="constE2")
      {
       Evals <- seq(from=input$ConstE.range[1],to=input$ConstE.range[2],length=10)
       for (II in 1:10)
        { 
         isolate(values$Outs <- BreakCtl(Parms,input$HCR.range[1],input$HCR.range[2],input$HCR.prop2,"constE",-1,Evals[II],input$Nsim))
         a <- CalculateOutputs(values$Outs,input$Nyear,input$BLIM,input$CLIM)
         newLine <- isolate(c("Constant Exploitation",NA,NA,NA,NA,round(Evals[II],3),round(a$PercC[2],1),round(a$PercB[2],1),a$ProbGreen,a$ProbSafe,a$ProbBLIM,a$ProbCLIM,a$AAV))
         isolate(values$df <- rbind(values$df, newLine))
         isolate(values$BioTraj <- rbind(values$BioTraj,values$Outs$BiomassOut) )
         isolate(values$CatTraj <- rbind(values$CatTraj,values$Outs$CatchOut) )
         newLine <- isolate(c(a$PercB,a$PercC,a$PercE))
         isolate(values$full <- rbind(values$full, newLine))
         NumberOfOptions <- NumberOfOptions + 1
       }
      } 
     if (input$HCR2.type=="constC2")
      {
       Evals <- seq(from=input$ConstC.range[1],to=input$ConstC.range[2],length=10)
       for (II in 1:10)
       { 
        isolate(values$Outs <- BreakCtl(Parms,input$HCR.range[1],input$HCR.range[2],input$HCR.prop2,"constC",Evals[II],-1,input$Nsim))
        a <- CalculateOutputs(values$Outs,input$Nyear,input$BLIM,input$CLIM)
        newLine <- isolate(c("Constant Catch",NA,NA,NA,round(Evals[II],0),NA,round(a$PercC[2],1),round(a$PercB[2],1),a$ProbGreen,a$ProbSafe,a$ProbBLIM,a$ProbCLIM,a$AAV))       
        isolate(values$df <- rbind(values$df, newLine))
        isolate(values$BioTraj <- rbind(values$BioTraj,values$Outs$BiomassOut) )
        isolate(values$CatTraj <- rbind(values$CatTraj,values$Outs$CatchOut) )
        newLine <- isolate(c(a$PercB,a$PercC,a$PercE))
        isolate(values$full <- rbind(values$full, newLine))
        NumberOfOptions <- NumberOfOptions + 1
       } 
      } 
   })   
     
  # do Manual projection
   observeEvent(input$ProjectUpdate,{
     finalY <-min(values$Single$Cyear+input$CLNyear,input$Nyear)
     for (Iyear in (values$Single$Cyear+1):finalY)
      {
       isolate(values$Single <- UpdateDynamics(values$Single,input$CL,Iyear))
      } 
     isolate(values$Single$Cyear <- finalY)
   })

   # Save plots as a PDF
   observeEvent(input$saveplot,{
     isolate(PDFoutputs(input$FileNameOut,values$full,values$BioTraj,values$CatTraj,input$OutputOption))
   })
     
   # Crappy test file
   output$estbiomass <- renderPlot({ DoPlots(values$Single,values$Single$Cyear) })
   output$truebiomass <- renderPlot({ FinalPlots(values$Single,values$Single$Cyear) })
   
   # tab for summary
   output$table1 <- renderTable({values$df[-1,c(1:6,8,9,10,11)]})
   # tab for summary
   output$table2 <- renderTable({values$df[-1,c(1:6,7,12,13)]})
   # plot of statustics
   output$plotOut <- renderPlot({ if (length(values$full$LowB) > 1) TradeOffPlots(values$full,values$BioTraj,values$CatTraj,input$OutputOption,Nyear=input$Nyear) },width=700,height=700)
   output$NULLS <- renderPlot({ plot(0,0,type="n",axes=F,xlab="",ylab="") })
   output$NULLS2 <- renderPlot({ plot(0,0,type="n",axes=F,xlab="",ylab="") })
   output$NULLS3 <- renderPlot({ plot(0,0,type="n",axes=F,xlab="",ylab="") })
   
   output$ParDisiplay1 <- renderPlot({ 
     values$Selex <- Selex(MidLen,input$Sel50,input$Delta)
     values$Retain <- Selex(MidLen,input$Ret50,input$Delta2)
     values$Growth <- Growth(MidLen,EndLen,max(1,input$GrowInc1),max(0.05,input$GrowInc2),max(0.01,input$Molt1),input$Molt2,SigmaV=input$SigmaL)
     Recs <- c(1,rep(0,length=nlen-1))
     values$Eqn <- EqnF(MidLen,values$Growth$TransX,Recs,values$Selex,values$Retain,M=input$Mval,Fval=input$Fval,LWE=input$LWE,Lmat=input$Lmat)
     Keyplots1(MidLen,values$Selex,values$Retain,values$Growth,values$Eqn)
     },width=600,height=600)

    output$ParDisiplay2 <- renderPlot({ 
      values$Selex <- Selex(MidLen,input$Sel50,input$Delta)
      values$Retain <- Selex(MidLen,input$Ret50,input$Delta2)
      values$Growth <- Growth(MidLen,EndLen,max(1,input$GrowInc1),max(0.05,input$GrowInc2),max(0.01,input$Molt1),input$Molt2,SigmaV=input$SigmaL)
      Recs <- c(1,rep(0,length=nlen-1))
      values$Eqn <- EqnF(MidLen,values$Growth$TransX,Recs,values$Selex,values$Retain,M=input$Mval,Fval=input$Fval,LWE=input$LWE,Lmat=input$Lmat)
     Keyplots2(MidLen,values$Selex,values$Retain,values$Growth,values$Eqn)
   },width=600,height=600)

    output$ParDisiplay3 <- renderPlot({ 
      values$Selex <- Selex(MidLen,input$Sel50,input$Delta)
      values$Retain <- Selex(MidLen,input$Ret50,input$Delta2)
      values$Growth <- Growth(MidLen,EndLen,max(1,input$GrowInc1),max(0.05,input$GrowInc2),max(0.01,input$Molt1),input$Molt2,SigmaV=input$SigmaL)
      Recs <- c(1,rep(0,length=nlen-1))
      Yields(MidLen,values$Growth$TransX,Recs,values$Selex,values$Retain,M=input$Mval,InitFval=FvalF,LWE=input$LWE,Lmat=input$Lmat,Steep=input$Steep,Fmax=input$Fmax)
      
    },width=600,height=600)
    
    output$ParDisiplay4 <- renderPlot({ 
      values$Selex <- Selex(MidLen,input$Sel50,input$Delta)
      values$Retain <- Selex(MidLen,input$Ret50,input$Delta2)
      values$Growth <- Growth(MidLen,EndLen,max(1,input$GrowInc1),max(0.05,input$GrowInc2),max(0.01,input$Molt1),input$Molt2,SigmaV=input$SigmaL)
      Recs <- c(1,rep(0,length=nlen-1))
      AllProject(MidLen,values$Growth$TransX,Recs,values$Selex,values$Retain,M=input$Mval,InitFval=FvalF,LWE=input$LWE,Lmat=input$Lmat,
                    Steep=input$Steep,SigmaR=input$SigmaR,prowR=input$ProwR,SigmaB=input$SigmaB,prowB=input$ProwB,
                    InitDepl=input$Init.Depl,SSBtarg=input$HCR.expl,LowB=input$HCR.range[1],HighB=input$HCR.range[1],
                    HCRType=input$HCR.type,Nsim=20)
      par(mfrow=c(3,2))

    },width=600,height=600)
    
    output$glossary <- renderTable({glossary})
    
    
       
}  # End of ShinyServer
## ------------------------------------------------------------------------------------ ##










